@@ -144,6 +144,10 @@ class SocialAuthBackend(object):
                 out.update(result)
             else:
                 return result
+
+        # clean the partial pipeline at the end of the process
+        if 'request' in kwargs:
+            clean_partial_pipeline(kwargs['request'])
         return out
 
     def extra_data(self, user, uid, response, details):
@@ -221,6 +225,7 @@ class OAuthBackend(SocialAuthBackend):
                 name, alias, discard = entry
             elif len(entry) == 1:
                 name = alias = entry
+                discard = False
             else:  # ???
                 continue
 
@@ -412,13 +417,20 @@ class BaseAuth(object):
         """
         name = self.AUTH_BACKEND.name
         if UserSocialAuth.allowed_to_disconnect(user, name, association_id):
+            do_revoke = setting('SOCIAL_AUTH_REVOKE_TOKENS_ON_DISCONNECT')
+            filter_args = {}
+
             if association_id:
-                UserSocialAuth.get_social_auth_for_user(user)\
-                                .get(id=association_id).delete()
+                filter_args['id'] = association_id
             else:
-                UserSocialAuth.get_social_auth_for_user(user)\
-                                .filter(provider=name)\
-                                .delete()
+                filter_args['provider'] = name
+            instances = UserSocialAuth.get_social_auth_for_user(user)\
+                                      .filter(**filter_args)
+
+            if do_revoke:
+                for instance in instances:
+                    instance.revoke_token(drop_token=False)
+            instances.delete()
         else:
             raise NotAllowedToDisconnect()
 
@@ -534,7 +546,8 @@ class OpenIdAuth(BaseAuth):
         """Return true if openid request will be handled with redirect or
         HTML content will be returned.
         """
-        return self.openid_request().shouldSendRedirect()
+        return self.openid_request(self.auth_extra_arguments())\
+                        .shouldSendRedirect()
 
     def openid_request(self, extra_params=None):
         """Return openid request"""
@@ -725,6 +738,8 @@ class BaseOAuth2(BaseOAuth):
     AUTHORIZATION_URL = None
     ACCESS_TOKEN_URL = None
     REFRESH_TOKEN_URL = None
+    REVOKE_TOKEN_URL = None
+    REVOKE_TOKEN_METHOD = 'POST'
     RESPONSE_TYPE = 'code'
     REDIRECT_STATE = True
     STATE_PARAMETER = True
@@ -855,9 +870,40 @@ class BaseOAuth2(BaseOAuth):
             data=urlencode(cls.refresh_token_params(token)),
             headers=cls.auth_headers()
         )
-        return cls.process_refresh_token_response(
-            dsa_urlopen(request).read()
-        )
+        return cls.process_refresh_token_response(dsa_urlopen(request).read())
+
+    @classmethod
+    def revoke_token_params(cls, token, uid):
+        return None
+
+    @classmethod
+    def revoke_token_headers(cls, token, uid):
+        return None
+
+    @classmethod
+    def process_revoke_token_response(cls, response):
+        return response.code == 200
+
+    @classmethod
+    def revoke_token(cls, token, uid):
+        if not cls.REVOKE_TOKEN_URL:
+            return
+        url = cls.REVOKE_TOKEN_URL.format(token=token, uid=uid)
+        params = cls.revoke_token_params(token, uid) or {}
+        headers = cls.revoke_token_headers(token, uid) or {}
+        data = None
+
+        if cls.REVOKE_TOKEN_METHOD == 'GET':
+            url = '{}?{}'.format(url, urlencode(params))
+        else:
+            data = urlencode(params)
+
+        request = Request(url, data=data, headers=headers)
+        if cls.REVOKE_TOKEN_URL.lower() not in ('get', 'post'):
+            # Patch get_method to return the needed method
+            request.get_method = lambda: cls.REVOKE_TOKEN_METHOD
+        response = dsa_urlopen(request)
+        return cls.process_revoke_token_response(response)
 
     def do_auth(self, access_token, *args, **kwargs):
         """Finish the auth process once the access_token was retrieved"""
